//
//  MixerAVListener.m
//  nice2meet
//
//  Created by Netease on 15/7/31.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "AVExportMixer.h"
#import "N2MSetting.h"
#import "avd_sdk/AVDAudio.h"
#import "avd_sdk/AVDVideo.h"
#import "avd_sdk/AVDCamera.h"
#import "avd_sdk/AVDFakeVideoCapturer.h"
#import "avd_sdk/AVDI420Frame.h"
#import "CoreVideo/CVPixelBuffer.h"

@interface AVExportMixer()<AVDAudioMixerDataDelegate,AVDVideoRendererCallBacks, AVDFakeVideoCapturerDelegate>
@property (nonatomic,retain) AVDVideo *mvideo;
@property (nonatomic,retain) AVDAudio *maudio;

@property (assign, nonatomic) BOOL islistenning;
//video overlays
@property (assign, nonatomic) CGFloat x;
@property (assign, nonatomic) CGFloat w;
@property (assign, nonatomic) CGFloat h;
@property (assign, nonatomic) CGFloat curr_y;
@property (assign, nonatomic) CGFloat curr_y_begin;
@property (strong, nonatomic) NSMutableArray *curr_overlays;
//audio buffer
@property (assign, nonatomic) NSInteger aac_audio_frame;
@property (assign, nonatomic) NSInteger cur_audio_frame;

// for fake video capturer
@property (nonatomic,retain) AVDFakeVideoCapturer *mfakeVideo;
@property (nonatomic,retain) NSString *mfakeDeviceId;
@property (nonatomic,retain) NSMutableData *mvideobuff;
@property (assign, nonatomic) NSInteger ylen;
@property (assign, nonatomic) NSInteger ulen;
@property (assign, nonatomic) NSInteger vlen;

@end

@implementation AVExportMixer

- (instancetype)init {
    self = [super init];
    if (self) {
        AVDRoom* room = [AVDRoom obtain:[N2MSetting getCurrentRoomId]];
        self.mvideo = [AVDVideo getVideo:room];
        self.maudio = [AVDAudio getAudio:room];
        
        self.aac_audio_frame = 2048; // aac 1024 sample mono channel buffer size;
        self.aac_audio_frame = 882;  // 44100 samplerate 10ms buffer size;
        [self resetMembers];
        
        self.mfakeVideo = [[AVDFakeVideoCapturer alloc] initWithDelegate:self fourFormat:ft_I420 isScreen:NO];
        self.mfakeDeviceId = @"fakeVid";
    }
    return self;
}

- (BOOL) listenData {
    if (self.islistenning) {
        return YES;
    }
    NSLog(@"MixerAVListener, listenData begin...");
    [self.mvideo setMixerDateListener:self];
    BOOL hasMainVideo = NO;
    for (AVDCamera* item in self.mvideo.publishedCameras) {
        if ([self.mvideo isSelfDevice:item.id]) {
            [self.mvideo setMixerMainVideo:item.id];
            hasMainVideo = YES;
            break;
        }
    }
    if (hasMainVideo) {
        for (AVDCamera* item in self.mvideo.publishedCameras) {
            if (![self needOverlayVideo]) {
                break;
            }
            if ([self.mvideo isSelfDevice:item.id]) {
                continue;
            }
            [self addOverlayVideo:item.id];
        }
    }
    else {
        NSLog(@"MixerAVListener, listenData, no main video, cann't start live.");
        [self.mvideo setMixerDateListener:nil];
        return NO;
    }

    self.maudio.mixerDataDelegate = self;
    self.islistenning = YES;
    
    if (hasMainVideo && Nil != self.mfakeVideo) {
        AVDCamera* fakeCam = [[AVDCamera alloc] initWithDeviceId:self.mfakeDeviceId name:@"fake camera"];
        [self.mvideo publishLocalCamera:fakeCam capturer:self.mfakeVideo];
    }
    NSLog(@"MixerAVListener, listenData ok.");
    return YES;
}

- (BOOL) listenDataWithVideoId:(AVDDeviceId)vid {
    if (self.islistenning) {
        return YES;
    }
    NSLog(@"MixerAVListener, listenDataWithVideoId begin...");
    [self.mvideo setMixerDateListener:self];
    [self.mvideo setMixerMainVideo:vid];
    
    self.maudio.mixerDataDelegate = self;
    self.islistenning = YES;
    
    if (Nil != self.mfakeVideo) {
        AVDCamera* fakeCam = [[AVDCamera alloc] initWithDeviceId:self.mfakeDeviceId name:@"fake camera"];
        [self.mvideo publishLocalCamera:fakeCam capturer:self.mfakeVideo];
    }
    NSLog(@"MixerAVListener, listenData ok.");
    return YES;
}

- (void) stopListen {
    if (!self.islistenning) {
        return ;
    }
    //
    NSLog(@"MixerAVListener, stopListen");
    [self.mvideo setMixerDateListener:nil];
    [self.mvideo clearMixerVideos];
     self.maudio.mixerDataDelegate = nil;
    //
    [self resetMembers];
}

- (BOOL) needOverlayVideo {
    return self.curr_y <(1.0-self.h);
}

//
- (void) resetMembers {
    self.islistenning = NO;
    self.x = 0.7;
    self.w = 0.3;
    self.h = 0.3;
    self.curr_y_begin = 0.1;
    self.curr_y = self.curr_y_begin;
    if (nil == self.curr_overlays) {
        self.curr_overlays = [[NSMutableArray alloc] init];
    }
    [self.curr_overlays removeAllObjects];
}

- (void)addOverlayVideo:(NSString*)fromId {
    NSLog(@"MixerAVListener, addOverlayVideo");
    [self.mvideo addMixerOverlayVideo:fromId zorder:0 x:self.x y:self.curr_y w:self.w h:self.h];
    [self.curr_overlays addObject:fromId];
    self.curr_y += self.h;
}
- (void)relayoutOverlayVideos {
    NSLog(@"MixerAVListener, relayoutOverlayVideos");
    self.curr_y = self.curr_y_begin;
    for (NSString* item in self.curr_overlays) {
        [self.mvideo updateMixerOverlayVideo:item zorder:0 x:self.x y:self.curr_y w:self.w h:self.h];
        self.curr_y += self.h;
    }
}
- (BOOL)overlayContainVideo:(NSString*)videoId {
    for (NSString* item in self.curr_overlays) {
        if ([item isEqualToString:videoId]) {
            return YES;
        }
    }
    return NO;
}


#pragma mark --audio data delegate 音频数据回调
/** 此函数在混音线程中回调上来，每次回调的音频数据为10ms的数据长度cur_audio_frame； 
    应用层不能做复杂耗时操作，只拷贝数据后立即返回
 */
- (void) onAudioData:(NSData*)data pts:(NSInteger)pts {
    //NSLog(@"MixerAVListener, onAudioData data:%lu pts:%ld", (unsigned long)data.length, (long)pts);
    // 将数据放入循环缓冲区中；
    
    // 如果循环缓冲区中数据量大于aac_audio_frame，放入aac编码缓冲区去编码；同时在循环缓冲中移除aac_audio_frame长度数据；
    
}
- (void) onAudioParam:(NSUInteger)sampleRate channels:(NSUInteger)channels {
    NSLog(@"MixerAVListener, onAudioParam sampleRate:%lu  channels:%lu", (unsigned long)sampleRate, (unsigned long)channels);
}

#pragma mark --video data delegate 视频数据回调
- (void)setSize:(CGSize)size {
    NSLog(@"MixerAVListener, setSize size:%f x %f", size.width, size.height);
    if (nil == self.mvideobuff || self.mvideobuff.length != [self calcVideoBuffLen:size]) {
        self.mvideobuff = [NSMutableData dataWithLength: [self calcVideoBuffLen:size] ];
    }
}
/** 此函数在视频渲染线程中回调上来，应用层尽量不做复杂耗时操作；
 */
- (void)renderFrame:(AVDI420Frame*)frame {
    if (nil == self.mvideobuff) {
        return ;
    }

    if (nil != self.mfakeVideo && [self.mfakeVideo isRunning]) {
        [self.mvideobuff replaceBytesInRange:NSMakeRange(0,self.ylen) withBytes:frame.yPlane length:self.ylen];
        [self.mvideobuff replaceBytesInRange:NSMakeRange(self.ylen,self.ulen) withBytes:frame.uPlane length:self.ulen];
        [self.mvideobuff replaceBytesInRange:NSMakeRange(self.ylen+self.ulen,self.vlen) withBytes:frame.vPlane length:self.vlen];
        [self.mfakeVideo inputCapturedFrame:self.mvideobuff w:frame.width h:frame.height rotation:0];
    }
}

- (NSUInteger)calcVideoBuffLen:(CGSize)size {
    self.ylen = size.height * size.width;
    self.ulen = size.height * size.width/4;
    self.vlen = self.ulen;
    return size.height * size.width * 3/2;
}

#pragma mark --video status delegate 视频状态回调
- (void)onVideo:(NSString*)fromId subscribed:(BOOL)subed {
    if (nil == self.mvideo || nil == fromId) {
        return ;
    }
    if ([self.mvideo isSelfDevice:fromId]) {
        return ;
    }
    if (subed) {
        if (![self needOverlayVideo]) {
            return ;
        }
        [self addOverlayVideo:fromId];
    }
    else {
        if (![self overlayContainVideo:fromId]) {
            return ;
        }
        [self.mvideo removeMixerOverlayVideo:fromId];
        [self.curr_overlays removeObject:fromId];
        [self relayoutOverlayVideos];
    }
}

#pragma mark --AVDFakeVideoCapturerDelegate 回调
- (BOOL) onStart {
    return YES;
}
- (void) onStop {
    
}

@end
