//
//  BaseRoomViewController.m
//  nice2meet
//
//  Created by skin on 16/3/18.
//  Copyright © 2016年 Netease. All rights reserved.
//

#import "BaseRoomViewController.h"
#import "N2MSetting.h"
#import "avd_sdk/AVDEngine.h"

@interface BaseRoomViewController ()<T3RoomToolbarDelegate, VideosViewGesture>
@property(weak, nonatomic) IBOutlet UILabel *clock_ui;
@property(weak, nonatomic) IBOutlet UILabel *roomid_ui;
@property(nonatomic, strong) NSTimer *clock_timer;
@property(nonatomic, assign) NSUInteger clock_second;

@end

@implementation BaseRoomViewController

- (void) viewDidLoad {
    [super viewDidLoad];
    if (nil == self.room) {
        NSLog(@"BaseRoomViewController, viewDidLoad, room is nil.");
        return ;
    }
    
    self.room.delegate = self;
    self.toolbar.delegate = self;
    self.videos_ui.delegate = self;
    self.clock_second = 0;
    self.clock_timer =  [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(updateWallTime) userInfo:nil repeats:YES];
    self.chats_ui.hidden = YES;
    self.users_ui.hidden = YES;
    self.roomid_ui.text = [NSString stringWithFormat:@"房间号%@",[self.room getRoomId]];
    
    if ([N2MSetting isAutoAudioOpen]) {
        [self performSelector:@selector(delayOnMicrophone) withObject:nil afterDelay:0.5];
    }
    if ([N2MSetting isAutoVideoOpen]) {
        [self performSelector:@selector(delayOnVideo) withObject:nil afterDelay:1.0];
    }
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [self.clock_timer invalidate];
}
- (void)delayOnMicrophone {
    [self.toolbar btnOnMicrophone:nil];
}
- (void)delayOnVideo {
    [self.toolbar btnOnVideo:nil];
}

#pragma mark --videos View delegate
- (void)onVideosViewTapped {
    self.toolbar.hidden = !self.toolbar.hidden;
}

#pragma mark --room toolbar delegate 房间工具条回调
- (BOOL)onLeaveRoom {
    [self.videos_ui stopLiveRecord];
    [self leaveRoom];
    return YES;
}
- (BOOL)onOpenCloseMicrophone {
    return [self.users_ui openCloseMicrophone];
}
- (BOOL)onOpenCloseLocacamera {
    return [self.videos_ui openCloseLocacamera];
}
- (BOOL)onShowHideChats {
    self.chats_ui.hidden = !self.chats_ui.hidden;
    return !self.chats_ui.hidden;
}
- (BOOL)onShowHideUsers {
    self.users_ui.hidden = !self.users_ui.hidden;
    return !self.users_ui.hidden;
}

- (void)leaveRoom {
    [self.room leave:AVD_Success];
     self.room.delegate = nil;
    [self dismissViewControllerAnimated:YES completion:^{
        [self.room destory];
    }];
    
    [[AVDEngine instance] uninit];
}

#define Format_Zero     @"0%lu"
#define Format_noZero   @"%lu"
- (void)updateWallTime {
    ++ self.clock_second;
    
    NSString *formatTime;
    NSString *hs, *ms, *ss;
    NSUInteger h, m, s;
    h = self.clock_second / 3600;
    m = (self.clock_second % 3600) / 60;
    s = (self.clock_second % 3600) % 60;
    hs = [NSString stringWithFormat:(h < 10) ? Format_Zero : Format_noZero,(unsigned long)h];
    ms = [NSString stringWithFormat:(m < 10) ? Format_Zero : Format_noZero,(unsigned long)m];
    ss = [NSString stringWithFormat:(s < 10) ? Format_Zero : Format_noZero,(unsigned long)s];
    formatTime = [NSString stringWithFormat:@"幸会 %@:%@:%@",hs,ms,ss];
    self.clock_ui.text = formatTime;
}

#pragma mark --room delegate 房间回调
- (void)onJoinResult:(AVDResult)result {
}
- (void)onLeaveIndication:(AVDResult)reason fromUser:(AVDUserId)fromId {
    NSLog(@"BaseRoomViewController, onLeaveIndication, reason:%lu", (unsigned long)reason);
    [self onLeaveRoom];
}
- (void)onPublicData:(NSData*)data  fromUser:(AVDUserId)fromId {
    NSString *sdata = [NSString stringWithUTF8String:(const char *)(data.bytes)];
    NSLog(@"BaseRoomViewController, onPublicData, data:%@, sdata:%@", data, sdata);
}
- (void)onPrivateData:(NSData*)data fromUser:(AVDUserId)fromId {
    NSString *sdata = [NSString stringWithUTF8String:(const char *)(data.bytes)];
    NSLog(@"BaseRoomViewController, onPrivateData, data:%@, sdata:%@", data, sdata);
}
- (void)onAppDataNotify:(NSString*)key value:(NSString*)value {
    NSLog(@"BaseRoomViewController, onAppDataNotify, value:%@", value);
}
- (void)onRoomStatusNotify:(enum AVDRoomStatus)status {
    NSLog(@"BaseRoomViewController, onRoomStatusNotify, status:%u", status);
}
- (void)onConnectionStatus:(enum AVDConnectionStatus)status {
    NSLog(@"BaseRoomViewController, onConnectionStatus, status:%u", status);
    if (connectFailed == status) {
        [self onLeaveRoom];
    }
}

@end
