//
//  ChatsView.m
//  nice2meet
//
//  Created by Netease on 15/8/11.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "ChatsView.h"
#import "N2MSetting.h"
#import "ChatContentCell.h"
#import "avd_sdk/AVDChat.h"

#define kInputMessage_placehoder             @"请输入消息"

@interface ChatsView()<AVDChatDelegate, UITextViewDelegate,UITableViewDataSource,UITableViewDelegate>
@property (nonatomic,strong) UITableView *messagesView;
@property (nonatomic,strong) UIView *inputView;
@property (nonatomic,strong) UITextView *textView;
@property (nonatomic,strong) UIButton *sendButton;
@property (nonatomic,strong) NSLayoutConstraint *chatViewbottom; //chatView距离底部的距离
//
@property (nonatomic,strong) ChatContentCell *dummy;
@property(nonatomic, weak) AVDChat* mchat;
@property(nonatomic, weak) AVDUser* selfUser;
@property(nonatomic,retain) NSMutableArray *messages;

@end

@implementation ChatsView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (void)initCommon {
    self.backgroundColor = [UIColor grayColor];
    if (nil == self.messagesView) {
        self.messagesView = [[UITableView alloc] initWithFrame:CGRectZero];
        self.messagesView.delegate = self;
        self.messagesView.dataSource = self;
        self.messagesView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
        [self addSubview:self.messagesView];
    }
    if (nil == self.inputView) {
        self.inputView = [[UIView alloc] initWithFrame:CGRectZero];
        self.inputView.backgroundColor = [UIColor clearColor];
        [self addSubview:self.inputView];
    }
    if (nil == self.textView) {
        self.textView = [[UITextView alloc] initWithFrame:CGRectZero];
        self.textView.text = kInputMessage_placehoder;
        self.textView.font = [UIFont systemFontOfSize:16.0f];
        [self.inputView addSubview:self.textView];
        [self.textView setDelegate:self];
    }
    if (nil == self.sendButton) {
        self.sendButton = [UIButton buttonWithType:UIButtonTypeRoundedRect];
        self.sendButton.backgroundColor = [UIColor clearColor];
        [self.sendButton setTitleColor:[UIColor blackColor]forState:UIControlStateNormal];
        [self.sendButton setTitle:@"发送" forState:UIControlStateNormal];
        [self.inputView addSubview:self.sendButton];
        [self.sendButton addTarget:self action:@selector(btnSendChatMessage:) forControlEvents:UIControlEventTouchUpInside];
    }
    self.messagesView.translatesAutoresizingMaskIntoConstraints = NO;
    self.inputView.translatesAutoresizingMaskIntoConstraints = NO;
    self.textView.translatesAutoresizingMaskIntoConstraints = NO;
    self.sendButton.translatesAutoresizingMaskIntoConstraints = NO;
    NSDictionary *views = [NSDictionary dictionaryWithObjectsAndKeys:
                           self.messagesView, @"m",
                           self.inputView, @"i",
                           nil];
    [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"H:|[m]|" options:0 metrics:nil views:views]];
    [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"H:|[i]|" options:0 metrics:nil views:views]];
    [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(30)-[m][i(==42)]" options:0 metrics:nil views:views]];
    if (nil == self.chatViewbottom) {
        self.chatViewbottom = [NSLayoutConstraint constraintWithItem:self.inputView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self attribute:NSLayoutAttributeBottom multiplier:1 constant:0];
        [self addConstraint: self.chatViewbottom];
    }
    views = [NSDictionary dictionaryWithObjectsAndKeys:
                           self.textView, @"t",
                           self.sendButton, @"s",
                           nil];
    [self.inputView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"H:|-[t][s(==50)]-|" options:0 metrics:nil views:views]];
    [self.inputView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-[t]-|" options:0 metrics:nil views:views]];
    [self.inputView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-[s]-|" options:0 metrics:nil views:views]];
    
    UITapGestureRecognizer* tap = [[UITapGestureRecognizer alloc] init];
    [self.messagesView addGestureRecognizer:tap];
    [tap addTarget:self action:@selector(tapGesture:)];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHidden:) name:UIKeyboardWillHideNotification object:nil];
    self.dummy = [[ChatContentCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:nil];
    
    self.mchat = [AVDChat getChat:[AVDRoom obtain:[N2MSetting getCurrentRoomId]]];
    self.mchat.delegate = self;
    self.selfUser = self.mchat.usermanager.selfUser;
    self.messages = [self.mchat getPublicHistoryMessage:TRUE begin:0 count:20];
}
- (void)layoutSubviews {
    [super layoutSubviews];
    self.dummy.availableWidth = self.frame.size.width;
}

- (void)scrollTableToFoot:(BOOL)animated {
    NSInteger section = [self.messagesView numberOfSections];
    if (section<1) return;
    NSInteger row = [self.messagesView numberOfRowsInSection:section-1];
    if (row<1) return;
    NSIndexPath *ip = [NSIndexPath indexPathForRow:row-1 inSection:section-1];
    [self.messagesView scrollToRowAtIndexPath:ip atScrollPosition:UITableViewScrollPositionBottom animated:animated];
}

- (void)keyboardWillShow:(NSNotification *)notification {
    NSDictionary *userInfo = [notification userInfo];
    NSValue *animationDurationValue = [userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey];
    NSTimeInterval animationDuration;
    [animationDurationValue getValue:&animationDuration];
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:animationDuration];
    NSValue *value = [userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
    CGSize keyboardSize = [value CGRectValue].size;
    CGFloat keyboardHeight = keyboardSize.height;
    self.chatViewbottom.constant = -keyboardHeight;
    [UIView commitAnimations];
    NSLog(@"end");
}
- (void)keyboardWillHidden:(NSNotification *)notification {
    [self.textView resignFirstResponder];
    NSDictionary *userInfo = [notification userInfo];
    NSValue *animationDurationValue = [userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey];
    NSTimeInterval animationDuration;
    [animationDurationValue getValue:&animationDuration];
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:animationDuration];
    self.chatViewbottom.constant = 0.0f;
    [UIView commitAnimations];
}
- (void)textViewDidBeginEditing:(UITextView *)textView {
    if ([self.textView.text isEqualToString:kInputMessage_placehoder]) {
        self.textView.text = @"";
    }
}
- (void)textViewDidEndEditing:(UITextView *)textView {
    if (self.textView.text.length<1) {
        self.textView.text = kInputMessage_placehoder;
    }
}
- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text{
    return YES;
}
- (void)btnSendChatMessage:(id)sender {
    [self.textView resignFirstResponder];
    if (0 == [self.textView.text length]) {
        return ;
    }
    if ([self.textView.text isEqualToString:kInputMessage_placehoder]) {
        return ;
    }
    AVDMessage *msg = [[AVDMessage alloc] initWithFromId:self.selfUser.userId fromName:self.selfUser.userName message:self.textView.text];
    [self.messages addObject:msg];
    [self.mchat sendPublicMessage:self.textView.text];
    [self.textView setText:@""];
    [self.messagesView reloadData];
}

-(void)tapGesture:(UITapGestureRecognizer *)recognizer{
    [self.textView resignFirstResponder];
    [self resignFirstResponder];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark -
#pragma mark Table Data Source Methods
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.messages count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    //NSLog(@"ChatsView, cellForRowAtIndexPath, row:%ld", indexPath.row);
    static NSString *MessageCellIdentifier = @"MessageCellIdentifier";
    ChatContentCell *cell = [tableView dequeueReusableCellWithIdentifier:MessageCellIdentifier];
    if (cell == nil) {
        cell = [[ChatContentCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:MessageCellIdentifier];
        cell.userInteractionEnabled = NO;
        [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    }
    cell.availableWidth = self.frame.size.width;
    AVDMessage *msg = [self.messages objectAtIndex:indexPath.row];
    [cell showChatWithName:msg.fromName userId:msg.fromId message:msg.message isMe:[self.mchat isSelfUser:msg.fromId]];
    return cell;
}

#pragma mark Table Delegate Methods
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSLog(@"ChatsView, heightForRowAtIndexPath, row:%ld", indexPath.row);
    AVDMessage *msg = [self.messages objectAtIndex:indexPath.row];
    return [self.dummy getCellSize:msg.message].height;
}

#pragma mark --chat delegate 聊天回调方法
- (void) onPublicMessage:(AVDMessage*)message {
    [self.messages addObject:message];
    [self.messagesView reloadData];
    [self scrollTableToFoot:YES];
}
- (void) onPrivateMessage:(AVDMessage*)message {
    [self.messages addObject:message];
    [self.messagesView reloadData];
    [self scrollTableToFoot:YES];
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
