//
//  JoinningViewController.m
//  nice2meet
//
//  Created by Netease on 15/7/30.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "JoinningViewController.h"
#import "WaitingView.h"
#import "N2MSetting.h"
#import "avd_sdk/AVDEngine.h"
#import "avd_sdk/AVDRoom.h"
#import "PortraitRoomViewController.h"
#import "LandscapeRoomViewController.h"

@interface JoinningViewController ()<UITextFieldDelegate,AVDRoomJoinDelegate,WaitingViewDelegate, AVDEngineDelegate>
@property (weak, nonatomic) IBOutlet UITextField *meetingIDField;
@property (weak, nonatomic) IBOutlet UITextField *userNameField;
@property (weak, nonatomic) IBOutlet WaitingView *waitingView;
@end

@implementation JoinningViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.meetingIDField.returnKeyType = UIReturnKeyNext;
    self.userNameField.returnKeyType  = UIReturnKeyGo;
    self.meetingIDField.text = [N2MSetting getCurrentRoomId];
    self.userNameField.text = [N2MSetting getUserName];
    
    self.waitingView.delegate = self;
    [self.waitingView stopWaiting];
    UIDeviceOrientation oldorientation = [UIDevice currentDevice].orientation;
    NSLog(@"viewDidLoad, orientation: curr %lu", (unsigned long)oldorientation);
    [self setDeviceOrientation:UIDeviceOrientationPortrait];
}

- (void)setDeviceOrientation:(UIDeviceOrientation)orientation {
    NSLog(@"setDeviceOrientation, orientation: old %lu, new %lu", (unsigned long)[UIDevice currentDevice].orientation, (unsigned long)orientation);
    if ([[UIDevice currentDevice] respondsToSelector:@selector(setOrientation:)]) {
        [[UIDevice currentDevice] performSelector:@selector(setOrientation:)
                                       withObject:[NSNumber numberWithInteger:orientation]];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHidden:) name:UIKeyboardWillHideNotification object:nil];
}

- (void)viewDidDisappear:(BOOL)animated {
     [super viewDidDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];
}

- (void)keyboardWillShow:(NSNotification *)notification {
    NSDictionary *userInfo = [notification userInfo];
    NSValue *animationDurationValue = [userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey];
    NSTimeInterval animationDuration;
    [animationDurationValue getValue:&animationDuration];
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:animationDuration];
    
    NSValue *value = [userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
    CGSize keyboardSize = [value CGRectValue].size;
    CGFloat keyboardHeight = keyboardSize.height;
    CGRect selfBounds = self.view.bounds;
    selfBounds.origin.y = keyboardHeight/2;
    [self.view setBounds:selfBounds];
    [UIView commitAnimations];
    NSLog(@"end");
}

- (void)keyboardWillHidden:(NSNotification *)notification {
    NSDictionary *userInfo = [notification userInfo];
    NSValue *animationDurationValue = [userInfo objectForKey:UIKeyboardAnimationDurationUserInfoKey];
    NSTimeInterval animationDuration;
    [animationDurationValue getValue:&animationDuration];
    [UIView beginAnimations:nil context:NULL];
    [UIView setAnimationDuration:animationDuration];

    CGRect selfBounds = self.view.bounds;
    selfBounds.origin.y = 0;
    [self.view setBounds:selfBounds];
    [UIView commitAnimations];
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    if ([self.meetingIDField isFirstResponder]) {
        [self.userNameField becomeFirstResponder];
    } else if([self.userNameField isFirstResponder]) {
        if ([self.meetingIDField.text length]>0 && [self.userNameField.text length] >0) {
            [self btnJoinMeeting:nil];
        }
    }
    return YES;
}

- (IBAction)btnJoinMeeting:(id)sender {
//    if (![[AVDEngine instance] isWorking]) {
//        [N2MSetting showAlertViewWithTitle:@"提醒" message:@"引擎没有初始化完成，请稍候加房间" delegate:self cancelButtonTitle:@"知道了"];
//        return ;
//    }
    
    [self.waitingView startWaiting];
    [self.meetingIDField resignFirstResponder];
    [self.userNameField resignFirstResponder];
    
    [[AVDEngine instance] initWithServerUrl: [N2MSetting getServerURL] accessKey: [N2MSetting getAppKey] secretKey:[N2MSetting getSecretKey] delegate:self];
    
}

- (IBAction)hideKeyBoard:(id)sender {
    [self.meetingIDField resignFirstResponder];
    [self.userNameField resignFirstResponder];
    [self.view resignFirstResponder];
}

#pragma mark --
#pragma AVD... Delegate
- (void)onJoinResult:(AVDResult)result{
    [self.waitingView stopWaiting];
    NSLog(@"ViewController onJoinResult, result:%lu", (unsigned long)result);
    if (AVD_Success == result) {
        [N2MSetting setCurrentRoomId:self.meetingIDField.text];
        [N2MSetting setUserName:self.userNameField.text];
        
        UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
        BaseRoomViewController *roomvc = nil;
        NSString *uiStype = [N2MSetting getUIStype];
        if      ([uiStype isEqualToString:@"横屏"]) {
            roomvc = [storyboard instantiateViewControllerWithIdentifier:@"LandscapeRoomViewController"];
        }
        else if ([uiStype isEqualToString:@"竖屏"]) {
            roomvc = [storyboard instantiateViewControllerWithIdentifier:@"PortraitRoomViewController"];
        }
        roomvc.room = [AVDRoom obtain:self.meetingIDField.text];
        roomvc.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
        [self presentViewController:roomvc animated:YES completion:nil];
    }
    else{
        [N2MSetting showAlertViewWithTitle:@"警告" message:@"进入房间失败,请检查账号是否正确" delegate:self cancelButtonTitle:@"知道了"];
    }
}

- (void)onCloseWaitingView{
    AVDRoom* room = [AVDRoom obtain:self.meetingIDField.text];
    if (nil != room) {
        [room leave:AVD_Success];
    }
}

-(void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

///
#pragma mark --avdengine delegate avd引擎回调
- (void)onInitResult:(AVDResult)result {
    if (AVD_Success == result) {
        [[UIApplication sharedApplication] setIdleTimerDisabled:YES];
        [[AVDEngine instance] setOption:video_codec_priority value:[N2MSetting getVideoCodecOption]];
        [[AVDEngine instance] setOption:camera_capability_default value:[N2MSetting getVideoResolutionOption]];
        [[AVDEngine instance] setOption:data_channel_tcp_priority value:[N2MSetting getDataChannelNetOption]];
        NSLog(@"AppDelegate, AVDEngine onInitResult init success");
        
        
        NSString* roomId = self.meetingIDField.text;
        
        if ([roomId length]>0&& [self.userNameField.text length] >0) {
            AVDRoom* room = [AVDRoom obtain:roomId];
            //[room setOption:ro_media_use_dtls value:@"true"];
            AVDUser* user = [[AVDUser alloc] initWithUserId:[N2MSetting getUserId] userName:self.userNameField.text userData:nil];
            //[room joinWithUser:user password:nil delegate:self oemToken:@"FiteyApnqH602g1ndBo9aqIf-OzwQnuRLpXH2ABM:fNO9NJweJmkKTsa_xR_u4OqlkQo="];
            [room joinWithUser:user password:nil delegate:self];
        }
        else if ([self.meetingIDField.text length]<=0){
            [N2MSetting showAlertViewWithTitle:@"提醒" message:@"房间号不能为空" delegate:self cancelButtonTitle:@"知道了"];
        }
        else{
            [N2MSetting showAlertViewWithTitle:@"提醒" message:@"用户名不能为空" delegate:self cancelButtonTitle:@"知道了"];
        }

    }
    else {
        NSLog(@"AppDelegate, AVDEngine onInitResult init failed. result:%lu",(unsigned long)result);
    }
}
- (void)onUninitResult:(AVDResult)reason {
    NSLog(@"AppDelegate, AVDEngine onUninitResult reason:%lu",(unsigned long)reason);
}

@end
