//
//  T3RoomToolbar.m
//  nice2meet
//
//  Created by skin on 16/3/30.
//  Copyright © 2016年 Netease. All rights reserved.
//

#import "T3RoomToolbar.h"

@interface T3RoomToolbar()
@property (strong, nonatomic) UIButton *chats_btn;
@property (strong, nonatomic) UIButton *users_btn;
@property (strong, nonatomic) UIButton *leave_btn;
@property (strong, nonatomic) UIButton *camera_btn;
@property (strong, nonatomic) UIButton *microphone_btn;
@property (assign, nonatomic) BOOL chats_show;
@property (assign, nonatomic) BOOL users_show;

@end

@implementation T3RoomToolbar

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (void)initCommon {
    self.backgroundColor = [UIColor clearColor];
    if (nil == self.leave_btn) {
        self.leave_btn = [self addNextButtionWithImage:@"OnLine_btn_HangUp" action:@selector(btnOnLeave:)];
    }
    if (nil == self.users_btn) {
        self.users_btn = [self addNextButtionWithImage:@"icon_users_hide" action:@selector(btnOnUsers:)];
        self.users_show = NO;
    }
    if (nil == self.chats_btn) {
        self.chats_btn = [self addNextButtionWithImage:@"icon_chats_hide" action:@selector(btnOnChats:)];
        self.chats_show = NO;
    }
    if (nil == self.camera_btn) {
        self.camera_btn = [self addNextButtionWithImage:@"icon_video_hide" action:@selector(btnOnVideo:)];
    }
    if (nil == self.microphone_btn) {
        self.microphone_btn = [self addNextButtionWithImage:@"icon_audio_close" action:@selector(btnOnMicrophone:)];
    }
}
- (UIButton*)addNextButtionWithImage:(NSString*)image action:(SEL)action{
    UIButton *button = [[UIButton alloc] initWithFrame:CGRectZero];
    [self addSubview:button];
    [button addTarget:self action:action forControlEvents:UIControlEventTouchUpInside];
    [button setBackgroundImage:[UIImage imageNamed:image] forState:UIControlStateNormal];
    return button;
}
#define KPaddingBottom                  5.0f
#define KPaddingBetween                 6.0f
#define KLeaveWidth                     50.0f
#define KOtherWidth                     42.0f
- (void) layoutSubviews {
    [super layoutSubviews];
    CGFloat X = self.bounds.size.width, Y = self.bounds.size.height;
    if (self.landscape) {
        X = X-KPaddingBottom-KLeaveWidth;
        Y = (Y-KLeaveWidth)/2;
    }
    else {
        X = (X-KLeaveWidth)/2;
        Y = Y-KPaddingBottom-KLeaveWidth;
    }
    self.leave_btn.frame = CGRectMake(X, Y, KLeaveWidth, KLeaveWidth);
    CGFloat XY = 0;
    if (self.landscape) {
        Y = Y-KPaddingBetween-KOtherWidth;
        X = self.bounds.size.width-KPaddingBottom-KOtherWidth;
    }
    else {
        X = X-KPaddingBetween-KOtherWidth;
        Y = self.bounds.size.height-KPaddingBottom-KOtherWidth;
    }
    self.users_btn.frame = CGRectMake(X, Y, KOtherWidth, KOtherWidth);
    XY = (self.landscape) ? Y : X;
    XY = XY-KPaddingBetween-KOtherWidth;
    self.chats_btn.frame = CGRectMake((self.landscape ? X : XY), (self.landscape ? XY : Y), KOtherWidth, KOtherWidth);
    //
    XY = (self.landscape) ? self.leave_btn.frame.origin.y : self.leave_btn.frame.origin.x;
    XY = XY+KLeaveWidth+KPaddingBetween;
    self.camera_btn.frame = CGRectMake((self.landscape ? X : XY), (self.landscape ? XY : Y), KOtherWidth, KOtherWidth);
    XY = XY+KPaddingBetween+KOtherWidth;
    self.microphone_btn.frame = CGRectMake((self.landscape ? X : XY), (self.landscape ? XY : Y), KOtherWidth, KOtherWidth);
}

- (void)setLandScape:(BOOL)landscape {
    _landscape = landscape;
    [self setNeedsLayout];
}
- (IBAction)btnOnChats:(id)sender {
    if (self.users_show) {
        [self openCloseUsers];
    }
    [self openCloseChats];
}
- (void)openCloseUsers {
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onShowHideUsers)] ) {
        return;
    }
    self.users_show = [self.delegate onShowHideUsers];
    [self updateButtonImage:self.users_btn value:self.users_show icon1:@"icon_users_show" icon2:@"icon_users_hide"];
}
- (void)openCloseChats {
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onShowHideChats)] ) {
        return;
    }
    self.chats_show = [self.delegate onShowHideChats];
    [self updateButtonImage:self.chats_btn value:self.chats_show icon1:@"icon_chats_show" icon2:@"icon_chats_hide"];
}
- (IBAction)btnOnUsers:(id)sender {
    if (self.chats_show) {
        [self openCloseChats];
    }
    [self openCloseUsers];
}
- (IBAction)btnOnLeave:(id)sender {
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onLeaveRoom)]) {
        return;
    }
    [self.delegate onLeaveRoom];
}
- (IBAction)btnOnVideo:(id)sender {
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onOpenCloseLocacamera)]) {
        return;
    }
    BOOL isOpen = [self.delegate onOpenCloseLocacamera];
    [self updateButtonImage:self.camera_btn value:isOpen icon1:@"icon_video_show" icon2:@"icon_video_hide"];
}
- (IBAction)btnOnMicrophone:(id)sender {
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onOpenCloseMicrophone)]) {
        return;
    }
    BOOL isOpen = [self.delegate onOpenCloseMicrophone];
    [self updateButtonImage:self.microphone_btn value:isOpen icon1:@"icon_audio_open" icon2:@"icon_audio_close"];
}
- (void)updateButtonImage:(UIButton*)button value:(BOOL)value icon1:(NSString*)icon1 icon2:(NSString*)icon2 {
    if (value) {
        [button setBackgroundImage:[UIImage imageNamed:icon1] forState:UIControlStateNormal];
    } else {
        [button setBackgroundImage:[UIImage imageNamed:icon2] forState:UIControlStateNormal];
    }
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
