//
//  UserViewCell.m
//  nice2meet
//
//  Created by Netease on 15/8/10.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "UserViewCell.h"
#import "N2MSetting.h"

@interface UserViewCell()
@property (strong, nonatomic) UILabel *userName;
@property (strong, nonatomic) UIButton *audioLevel;
@property (strong, nonatomic) UIButton *audioStatus;
@property (strong, nonatomic) UIButton *videoStatus;
@property (strong, nonatomic) UIButton *screenStatus;

@end

@implementation UserViewCell

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (void)initCommon {
    self.backgroundColor = [UIColor grayColor];
    if (nil == self.userName) {
        self.userName = [[UILabel alloc] initWithFrame:CGRectZero];
        self.userName.font = [UIFont systemFontOfSize:14.0f];
        self.userName.textColor = [UIColor whiteColor];
        [self addSubview:self.userName];
    }
    if (nil == self.audioLevel) {
        self.audioLevel = [self addNextButtionWithImage:@"icon_audio_level_1"];
    }
    if (nil == self.audioStatus) {
        self.audioStatus = [self addNextButtionWithImage:@"icon_audio_status"];
    }
    if (nil == self.videoStatus) {
        self.videoStatus = [self addNextButtionWithImage:@"icon_video_status"];
    }
    if (nil == self.screenStatus) {
        self.screenStatus = [self addNextButtionWithImage:@"icon_screen_status"];
    }
}
- (UIButton*)addNextButtionWithImage:(NSString*)image {
    UIButton *button = [[UIButton alloc] initWithFrame:CGRectZero];
    button.hidden = YES;
    [self addSubview:button];
    [button setBackgroundImage:[UIImage imageNamed:image] forState:UIControlStateNormal];
    
    [button addTarget: self action:@selector(btnRemoteCmd:) forControlEvents:UIControlEventTouchUpInside];
    return button;
}
- (void) layoutSubviews {
    [super layoutSubviews];
    CGFloat X = self.bounds.origin.x, Y = self.bounds.origin.y+5;
    X += 20;
    self.userName.frame = CGRectMake(X, Y, 120, 28);
    X = self.bounds.size.width - 22 -15;
    self.screenStatus.frame = CGRectMake(X, 6, 22, 22);
    self.videoStatus.frame = CGRectMake(X-(22+5), 6, 22, 22);
    self.audioStatus.frame = CGRectMake(X-(22+5)*2, 6, 22, 22);
    self.audioLevel.frame = CGRectMake(X-(22+5)*3, 6, 22, 22);
}

- (void)showUser:(AVDUser *)user {
    self.user = user;
    self.userName.text = self.user.userName;
//    self.audioStatus.hidden = ![self.user.status isMicrophoneOn];
//    self.videoStatus.hidden = ![self.user.status isCameraOn];
    self.screenStatus.hidden = ![self.user.status isScreenOn];
    [self showButton:self.audioStatus a:[user.status isMicrophoneOn] imga:@"icon_audio_status" b:[user.status hasMicrophone] imgb:@"icon_audio_status_close"];
    [self showButton:self.videoStatus a:[user.status isCameraOn] imga:@"icon_video_status" b:[user.status hasCamera] imgb:@"icon_video_status_close"];
}

- (void)showButton:(UIButton *)btn a:(BOOL)a imga:(NSString *)imga b:(BOOL)b imgb:(NSString *)imgb {
    NSString *icon;
    if      (a) {
        icon = imga;
        btn.tag = 2;
    }
    else if (b) {
        icon = imgb;
        btn.tag = 1;
    }
    if (nil == icon) {
        btn.hidden = YES;
        btn.tag = 0;
    }
    else {
        btn.hidden = NO;
        [btn setBackgroundImage:[UIImage imageNamed:icon] forState:UIControlStateNormal];
    }
}

- (void)btnRemoteCmd:(id)sender {
    UIButton *button = sender;
    if (nil == button || 0 == button.tag) {
        return ;
    }
    NSString *type;
    if      (self.audioStatus == button) {
        type = @"audio";
    }
    else if (self.videoStatus == button) {
        type = @"video";
    }
    NSNumber *status = [NSNumber numberWithInteger:button.tag];
    NSDictionary *info = [NSDictionary dictionaryWithObjectsAndKeys:
                          type, @"type",
                          status, @"status",
                          self.user.userId, @"userId", nil];
    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationUserCellRemoteCommand object:self userInfo:info];
 
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
    [super setSelected:selected animated:animated];
    // Configure the view for the selected state
}

@end
