//
//  UsersView.m
//  nice2meet
//
//  Created by Netease on 15/7/31.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "UsersView.h"
#import "UserViewCell.h"
#import "N2MSetting.h"
#import <avd_sdk/avd_sdk.h>

@interface UsersView()<AVDUserManagerDelegate,AVDAudioDelegate, UITableViewDataSource,UITableViewDelegate>
@property(nonatomic,strong)UILabel *titleLabel;
@property(nonatomic,strong)UILabel *numLabel;
@property(nonatomic,strong)UITableView *usersView;
//
@property (nonatomic,retain) AVDUserManager *musers;
@property (nonatomic,retain) AVDAudio *maudio;

@end

@implementation UsersView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (void)initCommon {
    self.backgroundColor = [UIColor grayColor];
    if (nil == self.titleLabel) {
        self.titleLabel = [self addNextLabel:@"人员列表"];
        [self addSubview:self.titleLabel];
    }
    if (nil == self.numLabel) {
        self.numLabel = [self addNextLabel:@""];
        [self addSubview:self.numLabel];
    }
    if (nil == self.usersView) {
        self.usersView = [[UITableView alloc] initWithFrame:CGRectZero];
        [self.usersView setBackgroundView:nil];
        [self.usersView setBackgroundView:[[UIView alloc]init]];
        self.usersView.backgroundView.backgroundColor = [UIColor clearColor];
        self.usersView.backgroundColor = [UIColor clearColor];
        self.usersView.delegate = self;
        self.usersView.dataSource = self;
        self.usersView.tableFooterView = [[UIView alloc] initWithFrame:CGRectZero];
        [self addSubview:self.usersView];
    }
    self.musers = [AVDUserManager getUserManager:[AVDRoom obtain:[N2MSetting getCurrentRoomId]]];
    self.musers.delegate = self;
    self.maudio = [AVDAudio getAudio:[AVDRoom obtain:[N2MSetting getCurrentRoomId]]];
    self.maudio.delegate = self;
    self.numLabel.text = [NSString stringWithFormat:@"共%lu人",(unsigned long)[self.musers.participants count]];
}
- (UILabel*)addNextLabel:(NSString*)text {
    UILabel *label = [[UILabel alloc] initWithFrame:CGRectZero];
    label.font = [UIFont systemFontOfSize:20.0f];
    label.textColor = [UIColor whiteColor];
    label.textAlignment = NSTextAlignmentCenter;
    label.text = text;
    return label;
}
- (void)layoutSubviews {
    [super layoutSubviews];
    CGFloat X = self.bounds.origin.x, Y = self.bounds.origin.y;
    Y += 30;
    self.titleLabel.frame = CGRectMake(X,Y, self.bounds.size.width-100, 36);
    self.numLabel.frame = CGRectMake(self.bounds.size.width-100,Y, 100, 36);
    Y += (36+5);
    self.usersView.frame = CGRectMake(X,Y, self.bounds.size.width, self.frame.size.height-20-36-5);
//    NSLog(@"UsersView, layoutSubviews all:%f,%f, table:%f,%f, back:%f,%f,", self.frame.size.width, self.frame.size.height,self.usersView.frame.size.width, self.usersView.frame.size.height, self.usersView.backgroundView.frame.size.width,self.usersView.backgroundView.frame.size.height);
}

- (BOOL)openCloseMicrophone {
    BOOL isOpen = YES;
    if ([self.maudio isOpenMicrophone]) {
        [self.maudio closeMicrophone];
        isOpen = NO;
    } else {
        [self.maudio openMicrophone];

    }
    return isOpen;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.musers.participants count];
}
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *UserCellIdentifier = @"UserCellIdentifier";
    UserViewCell *cell = [tableView dequeueReusableCellWithIdentifier:UserCellIdentifier];
    if (cell == nil) {
        cell = [[UserViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:UserCellIdentifier];
    }
    [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    cell.userInteractionEnabled = YES;
    
    AVDUser* user = [self.musers.participants objectAtIndex:indexPath.row];
    [cell showUser:user];
    return cell;
}
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

#pragma mark --userManager delegate 用户管理模块回调
- (void)onUserJoinNotify:(AVDUser *)user {
    [self.usersView reloadData];
    self.numLabel.text = [NSString stringWithFormat:@"共%lu人",(unsigned long)[self.musers.participants count]];
    NSLog(@"UsersView, onUserJoinNotify user:%@,status:%@%@%@", user.userName, [user.status isMicrophoneOn]?@"a":@"", [user.status isCameraOn]?@"v":@"", [user.status isScreenOn]?@"d":@"");
}
- (void)onUserLeaveNotify:(AVDUser *)user {
    [self.usersView reloadData];
    self.numLabel.text = [NSString stringWithFormat:@"共%lu人",(unsigned long)[self.musers.participants count]];
    NSLog(@"UsersView, onUserLeaveNotify user:%@", user.userName);
}
- (void)onUserUpdateNotify:(AVDUser *)user {
}
- (void)onUserStatusNotify:(AVDUserStatus*)status fromUser:(AVDUserId)fromId {
    [self.usersView reloadData];
    NSLog(@"UsersView, onUserStatusNotify user:%@,status:%@%@%@", fromId, [status isMicrophoneOn]?@"a":@"", [status isCameraOn]?@"v":@"", [status isScreenOn]?@"d":@"");
}
- (void)onUserDataNotify:(NSString *)data fromUser:(AVDUserId)fromId {
}

#pragma mark --audio delegate 音频模块回调
- (void) onMicrophoneStatusNotify:(enum AVDDeviceStatus)status fromUser:(AVDUserId) fromId {
}
- (void) onAudioLevelMonitorNotify:(AVDAudioInfo*)info {
    
}
- (void) onOpenMicrophoneResult:(AVDResult)result {
}
- (void) onCloseMicrophoneResult:(AVDResult)result {
}

@end
