//
//  VideoViewCell.m
//  nice2meet
//
//  Created by Netease on 15/8/18.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "VideoViewCell.h"
#import "N2MSetting.h"

@interface VideoViewCell()
@property (strong, nonatomic)UILabel *title_ui;
@property (strong, nonatomic)UIButton *switch_ui;
@property (strong, nonatomic)UIButton *record_ui;
@property (strong, nonatomic)UIButton *lvcast_ui;
@property (strong, nonatomic)UIButton *lshare_ui;
@property (strong, nonatomic)UIImageView *image_ui;

@end

@implementation VideoViewCell

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (void)initCommon {
    if (nil == self.title_ui) {
        self.title_ui = [[UILabel alloc] initWithFrame:CGRectZero];
        [self addSubview:self.title_ui];
    }
    if (nil == self.switch_ui) {
        self.switch_ui = [[UIButton alloc] initWithFrame:CGRectZero];
        self.switch_ui.hidden = YES;
        [self addSubview:self.switch_ui];
        [self.switch_ui setBackgroundImage:[UIImage imageNamed:@"video_switch_camera_normal"] forState:UIControlStateNormal];
        [self.switch_ui setBackgroundImage:[UIImage imageNamed:@"video_switch_camera_pressed"] forState:UIControlStateHighlighted];
        [self.switch_ui addTarget:self action:@selector(btnVideoSwitch:) forControlEvents:UIControlEventTouchUpInside];
    }
    if (nil == self.record_ui) {
        self.record_ui = [[UIButton alloc] initWithFrame:CGRectZero];
        self.record_ui.hidden = YES;
        [self addSubview:self.record_ui];
        [self.record_ui setBackgroundImage:[UIImage imageNamed:@"video_record_off"] forState:UIControlStateNormal];
        [self.record_ui addTarget:self action:@selector(btnVideoRecord:) forControlEvents:UIControlEventTouchUpInside];
    }
    if (nil == self.lvcast_ui) {
        self.lvcast_ui = [[UIButton alloc] initWithFrame:CGRectZero];
        self.lvcast_ui.hidden = YES;
        [self addSubview:self.lvcast_ui];
        [self.lvcast_ui setBackgroundImage:[UIImage imageNamed:@"video_live_off"] forState:UIControlStateNormal];
        [self.lvcast_ui addTarget:self action:@selector(btnVideoLive:) forControlEvents:UIControlEventTouchUpInside];
    }
    if (nil == self.lshare_ui) {
        self.lshare_ui = [[UIButton alloc] initWithFrame:CGRectZero];
        self.lshare_ui.hidden = YES;
        [self addSubview:self.lshare_ui];
        [self.lshare_ui setBackgroundImage:[UIImage imageNamed:@"icon_share_qrcode"] forState:UIControlStateNormal];
        [self.lshare_ui addTarget:self action:@selector(btnLiveShare:) forControlEvents:UIControlEventTouchUpInside];
    }
    if (nil == self.image_ui) {
        self.image_ui = [[UIImageView alloc] initWithFrame:CGRectZero];
        [self addSubview:self.image_ui];
    }
    UITapGestureRecognizer* tap = [[UITapGestureRecognizer alloc] init];
    [self addGestureRecognizer:tap];
    [tap addTarget:self action:@selector(GestureRecognizerSelect:)];
}
- (void) layoutSubviews {
    [super layoutSubviews];
    self.title_ui.frame = CGRectMake(0, 0, 80, 17);
    self.switch_ui.frame = CGRectMake(self.bounds.size.width-46, 15, 44, 32);
    self.record_ui.frame = CGRectMake(self.bounds.size.width-46, 15+35, 44, 32);
    self.lvcast_ui.frame = CGRectMake(self.bounds.size.width-46, 15+70, 44, 32);
    self.lshare_ui.frame = CGRectMake(self.bounds.size.width-46, 15+105, 44, 32);
    self.image_ui.frame = self.bounds;
}

- (BOOL)hasVideo {
    return nil != self.deviceId && ![self.deviceId isEqualToString:@""];
}
- (BOOL)issVideo:(AVDDeviceId)deviceId {
    return [deviceId isEqualToString:self.deviceId];
}
- (void)setVideo:(AVDDeviceId)deviceId title:(NSString*)title isSwitch:(BOOL)isswitch isRecord:(BOOL)isRecord isLive:(BOOL)isLive{
    _deviceId = deviceId;
    self.tag = [deviceId hash];
    self.title_ui.text = title;
    self.switch_ui.hidden = !isswitch;
    self.record_ui.hidden = !isRecord;
    self.lvcast_ui.hidden = !isLive;
    self.lshare_ui.hidden = YES;
    self.image_ui.hidden = [self hasVideo];
}

- (void)setBackground:(NSString*)imagename {
    [self.image_ui setImage: [UIImage imageNamed:imagename]];
}

- (void)btnVideoSwitch:(id)sender{
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onSwitchLocalCamera:)]) {
        return;
    }
    [self.delegate onSwitchLocalCamera:self.deviceId];
}
- (void)btnVideoRecord:(id)sender{
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onRecordVideo:)]) {
        return;
    }
    BOOL on  = (nil == self.recordId);
    BOOL ret = [self.delegate onRecordVideo:self.deviceId];
    if (on && ret) {
        [self recorduiSetBkgImage:on];
    }
    else {
        [self setRecordId:nil];
    }
}
- (void)btnVideoLive:(id)sender{
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onLivebcVideo:)]) {
        return;
    }
    BOOL on  = (nil == self.liveId);
    BOOL ret = [self.delegate onLivebcVideo:self.deviceId];
    if (on && ret) {
        [self liveuiSetBkgImage:on];
    }
    else {
        [self setLiveId:nil];
    }
}
- (void)btnLiveShare:(id)sender{
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onLiveShare:)]) {
        return;
    }
    [self.delegate onLiveShare:self.liveId];
}
- (void)setLiveId:(AVDLiveId)liveId {
    [self liveuiSetBkgImage:(nil != liveId)];
     self.lshare_ui.hidden =(nil == liveId);
    _liveId = liveId;
}
- (void)setRecordId:(AVDRecordId)recordId {
    [self recorduiSetBkgImage:(nil != recordId)];
    _recordId = recordId;
}
- (void)liveuiSetBkgImage:(BOOL)on {
    NSString* imgname = on ? @"video_live_on" : @"video_live_off";
    [self.lvcast_ui setBackgroundImage:[UIImage imageNamed:imgname] forState:UIControlStateNormal];
}
- (void)recorduiSetBkgImage:(BOOL)on {
    NSString* imgname = on ? @"video_record_on" : @"video_record_off";
    [self.record_ui setBackgroundImage:[UIImage imageNamed:imgname] forState:UIControlStateNormal];
}

- (void)GestureRecognizerSelect:(UITapGestureRecognizer *)sender {
    if (nil == self.deviceId || [self.deviceId isEqualToString:@""]) {
        return ;
    }
    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationGestureRecognizerSelect object:self userInfo:[NSDictionary dictionaryWithObject:self.deviceId forKey:@"DeviceId"]];
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
