//
//  VideosScrollView.m
//  nice2meet
//
//  Created by Netease on 15/8/18.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "VideosScrollView.h"
#import "VideoViewCell.h"
#import "N2MSetting.h"
#import "avd_sdk/AVDVideo.h"

@interface VideosScrollView()
@property (nonatomic,weak) id<VideoViewCellDelegate> cellDelegate;
@property (nonatomic,retain) NSMutableArray *freeCells;
@property (nonatomic,assign) CGPoint pointLeftTop;

@end

@implementation VideosScrollView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (void)initCommon {
    [self setBackgroundColor:[UIColor colorWithRed:0.874 green:0.874 blue:0.874 alpha:1.0]];
    self.videoIds = [[NSMutableArray alloc] init];
    self.freeCells = [[NSMutableArray alloc] init];
    self.cellWidth = KVideoCellWidth+KVideoCellPadding;
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] init];
    [self addGestureRecognizer:pan];
    [pan addTarget:self action:@selector(onPanMe:)];
}

- (void) setDelegate4Cell:(id<VideoViewCellDelegate>) delegate {
    self.cellDelegate = delegate;
}
- (BOOL)hasVideos {
    return self.videoIds.count >0;
}
- (BOOL)hasFreeRender {
    return [self getVideoNum] <self.videoMax;
}
- (NSUInteger) getVideoNum {
    return self.videoIds.count;
}
- (VideoViewCell *)getFreeRenderForVideoId:(AVDDeviceId)videoId {
    VideoViewCell *cell = nil;
    if (self.freeCells.count >0) {
        cell = self.freeCells.firstObject;
        [self.freeCells removeObject:cell];
    }
    else if ([self getVideoNum] <self.videoMax) {
        cell = [[VideoViewCell alloc]initWithFrame:CGRectZero];
        cell.delegate = self.cellDelegate;
        [cell setBackground:@"ic_novideo"];
        [cell setScalingType:Scale_Aspect_Full];
    }
    [self addSubview:cell];
    [cell setFrame:CGRectMake(self.cellWidth*[self getVideoNum],0, KVideoCellWidth, KVideoCellHeight)];
    [self setContentSize:CGSizeMake(self.cellWidth*[self getVideoNum], self.frame.size.height)];
    [self.videoIds addObject:videoId];
    return cell;
}
- (VideoViewCell *)getRenderByVideoId:(AVDDeviceId)videoId {
    return (VideoViewCell*)[self viewWithTag:[videoId hash]];
}

- (void)removeCell:(VideoViewCell *)cell {
    NSString *deviceId = cell.deviceId;
    if (nil == deviceId || [deviceId isEqualToString:@""]) {
        NSLog(@"removeCell, deviceId is null");
        return ;
    }
    
    BOOL found = FALSE;
    NSUInteger location = 0;
    for(; location<[self.videoIds count]; ++location) {
        if ([deviceId isEqualToString:[self.videoIds objectAtIndex:location]]) {
            [self.videoIds removeObjectAtIndex:location];
            found = TRUE;
            break;
        }
    }
    if (!found) {
        NSLog(@"removeVideo, cann't find video. deviceId:%@", deviceId);
        return ;
    }
    
    [cell removeFromSuperview];
    [self.freeCells addObject:cell];
    [self relocationVideosLocation:location];
}

- (void)onPanMe:(UIPanGestureRecognizer *)pan {
    //NSLog(@"onPanVideos_up, pan.state=%ld",(long)pan.state);
    CGPoint p = [pan locationInView:self.superview];
    if (pan.state==UIGestureRecognizerStateBegan) {
        self.pointLeftTop=p;
    }else if (pan.state==UIGestureRecognizerStateChanged) {
        [self setFrame:CGRectMake(self.frame.origin.x+(p.x-self.pointLeftTop.x), self.frame.origin.y+(p.y-self.pointLeftTop.y), self.frame.size.width, self.frame.size.height)];
        self.pointLeftTop=p;
    }else if (pan.state==UIGestureRecognizerStateEnded) {
    }
}

- (void)relocationVideosLocation:(NSUInteger)location {
    for (NSUInteger i=location; i<[self.videoIds count]; ++i) {
        UIView *cell=[self viewWithTag:[[self.videoIds objectAtIndex:i] hash]];
        [cell setFrame:CGRectMake(i*self.cellWidth, 15, cell.frame.size.width, cell.frame.size.height)];
    }
    [self setContentSize:CGSizeMake(self.cellWidth*[self.videoIds count], self.frame.size.height)];
}

@end
