//
//  VideosView.m
//  nice2meet
//
//  Created by Netease on 15/7/31.
//  Copyright (c) 2015年 Netease. All rights reserved.
//

#import "VideosView.h"
#import "N2MSetting.h"
#import "VideoViewCell.h"
#import "VideosScrollView.h"
#import "avd_sdk/AVDEngine.h"
#import "avd_sdk/AVDVideo.h"
#import "avd_sdk/AVDScreen.h"
#import "avd_sdk/AVDRecordInfo.h"
#import "avd_sdk/AVDRecord.h"
#import "avd_sdk/AVDLiveInfo.h"
#import "avd_sdk/AVDLive.h"
#import "avd_sdk/AVDRoom.h"
#import "AVExportMixer.h"

@interface VideosView()<AVDVideoDelegate,AVDScreenDelegate,AVDRecordDelegate,AVDLiveDelegate,VideoViewCellDelegate>
@property (nonatomic,strong)VideoViewCell *video_bg;
@property (nonatomic,strong)VideosScrollView *videos_up;
@property (nonatomic,strong)UIWebView *webview;
//
@property (nonatomic,retain)AVDVideo *mvideo;
@property (nonatomic,retain)AVDScreen *mscreen;
@property (nonatomic,retain)AVDAudio *maudio;
//
@property (nonatomic,retain)AVDRecord *mrecord;
@property (nonatomic,retain)AVDLive *mlive;
@property (nonatomic,retain)AVDRecordInfo *curRecord;
@property (nonatomic,retain)AVDLiveInfo *curLive;
@property (nonatomic,strong)NSString *baseurl;
@property (nonatomic,assign)BOOL needLoad;
// modified for qiniu data callback
@property (nonatomic,retain)AVExportMixer *mixerav;
@property (nonatomic,assign)BOOL mixerTest;
@property (nonatomic,assign)BOOL mixer_preview;
@property (nonatomic,assign)BOOL video_previewed;

@end

@implementation VideosView

- (instancetype)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (instancetype)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self initCommon];
    }
    return self;
}
- (void)initCommon {
    self.backgroundColor = [UIColor redColor];
    if (nil == self.video_bg) {
        self.video_bg = [[VideoViewCell alloc] initWithFrame:CGRectZero];
        self.video_bg.delegate = self;
        [self addSubview:self.video_bg];
        [self.video_bg setScalingType:[N2MSetting getVideoShowStype]];
    }
    if (nil == self.videos_up) {
        self.videos_up = [[VideosScrollView alloc] initWithFrame:CGRectZero];
        self.videos_up.hidden = YES;
        self.videos_up.videoMax = 1;
        [self.videos_up setDelegate4Cell:self];
        [self addSubview:self.videos_up];
    }
    if (nil == self.webview) {
        self.webview = [[UIWebView alloc] initWithFrame:CGRectZero];
        self.webview.hidden = YES;
        [self addSubview:self.webview];
    }
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] init];
    [self.video_bg addGestureRecognizer:tap];
    [tap addTarget:self action:@selector(onTapVideo_bg:)];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(switch2BigRender:) name:NSNotificationGestureRecognizerSelect object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(userRemoteCmd:) name:NSNotificationUserCellRemoteCommand object:nil];
    
    self.baseurl = @"http://";
    NSString* surl = [[AVDEngine instance] getOption:demo_urlbase_liverecord];
    self.baseurl = [self.baseurl stringByAppendingString:surl];
    self.needLoad = YES;
    
    AVDRoom *room = [AVDRoom obtain:[N2MSetting getCurrentRoomId]];
    self.mvideo = [AVDVideo getVideo:room];
    self.mvideo.delegate = self;
    self.mscreen = [AVDScreen getScreen:room];
    self.mscreen.delegate = self;
    self.maudio = [AVDAudio getAudio:room];
    [self performSelector:@selector(defaultSelectVideo4Show) withObject:self afterDelay:2];
    self.mrecord = [AVDRecord instance];
    self.mrecord.delegate = self;
    self.mlive = [AVDLive instance];
    self.mlive.delegate = self;
    // modified for qiniu data callback
    self.video_previewed = NO;
    self.mixerTest = NO;
    self.mixer_preview = NO;
    if (self.mixerTest) {
        self.mixerav = [[AVExportMixer alloc] init];
        self.videostatusdelegate = self.mixerav;
    }
}
- (void)layoutSubviews {
    [super layoutSubviews];
    self.video_bg.frame = self.bounds;
    if ([self.videos_up hasVideos]) {
        self.videos_up.frame = CGRectMake(self.videos_up.frame.origin.x+5, self.videos_up.frame.origin.y+10, self.videos_up.cellWidth*[self.videos_up getVideoNum], KVideoCellHeight);
        self.videos_up.hidden = NO;
    }
    else {
        self.videos_up.hidden = YES;
    }
    NSString *uiStype = [N2MSetting getUIStype];
    if      ([uiStype isEqualToString:@"横屏"]) {
        CGFloat webXY = self.bounds.origin.x + 60;
        self.webview.frame = CGRectMake(webXY, self.bounds.origin.y, self.bounds.size.width*2/3,self.bounds.size.height);
    }
    else if ([uiStype isEqualToString:@"竖屏"]) {
        CGFloat webXY = self.bounds.origin.y + self.bounds.size.height/3 - 60;
        self.webview.frame = CGRectMake(self.bounds.origin.x, webXY, self.bounds.size.width,self.bounds.size.height*2/3);
    }
}

- (void)onTapVideo_bg:(UITapGestureRecognizer *)tap {
    if (nil == self.delegate || ![self.delegate respondsToSelector:@selector(onVideosViewTapped)]) {
        return;
    }
    if (NO == self.webview.hidden) {
        self.webview.hidden = YES;
    }
    [self.delegate onVideosViewTapped];
}

#pragma mark --VideoViewCellDelegate delegate imp
- (void)onSwitchLocalCamera:(AVDDeviceId)deviceId {
    if (![self.mvideo isSelfDevice:deviceId]) {
        return ;
    }
    [self.mvideo switchToLocalCamera];
}
- (BOOL)onRecordVideo:(AVDDeviceId)videoId {
    AVDResult ret = AVD_Failture;
    if (nil == self.curRecord) {
        self.curRecord = [[AVDRecordInfo alloc] init];
        self.curRecord.name = [self getRecOrLiveName:videoId type:@"_rec_"];
        self.curRecord.userId = [self.mvideo getOwnerId:videoId];
        self.curRecord.roomId = [N2MSetting getCurrentRoomId];
        self.curRecord.audioType = ra_user_single;
        self.curRecord.videoType = [self getH264Stream:videoId];
        ret = [self.mrecord createUserRecord:self.curRecord];
    }
    else {
        ret = [self.mrecord stopRecord:self.curRecord.id];
        ret = AVD_Success;
        self.curRecord = nil;
    }
    return (AVD_Success == ret);
}
- (BOOL)onLivebcVideo:(AVDDeviceId)videoId {
    AVDResult ret = AVD_Failture;
    if (nil == self.curLive) {
        self.curLive = [[AVDLiveInfo alloc] init];
        self.curLive.name = [self getRecOrLiveName:videoId type:@"_liv_"];
        self.curLive.userId = [self.mvideo getOwnerId:videoId];
        self.curLive.roomId = [N2MSetting getCurrentRoomId];
        self.curLive.audioType = ra_user_single;
        self.curLive.videoType = [self getH264Stream:videoId];
        ret = [self.mlive createUserLive:self.curLive];
    }
    else {
        ret = [self.mlive deleteLive:self.curLive.id];
        ret = AVD_Success;
        self.curLive = nil;
    }
    return (AVD_Success == ret);
}
- (void)onLiveShare:(AVDLiveId)liveId {
    if (nil == self.baseurl || nil == self.curLive || nil == self.curLive.hlsurl) {
        return ;
    }
    if (!self.needLoad) {
        self.webview.hidden = NO;
        return ;
    }
    NSString* surl = self.baseurl;
    surl = [surl stringByAppendingString:@"/admin/live/qrcode.html?src="];
    surl = [surl stringByAppendingString:self.baseurl];
    surl = [surl stringByAppendingString:@"/admin/live/hlsPlayer.html?src="];
    surl = [surl stringByAppendingString:self.curLive.hlsurl];
    [self.webview loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:surl]]];
    self.webview.hidden = NO;
    self.needLoad = NO;
}

- (BOOL)openCloseLocacamera {
    BOOL isOpen = YES;
    // for test
    AVDResult ret = AVD_Success;
    if (self.mixer_preview) {
        if (self.video_previewed) {
            AVDDeviceId curid = [self.mvideo getCamera: [self.mvideo getCurrentCameraType] ].id;
            ret = [self.mvideo unpreviewLocalCamera:curid];
            self.video_previewed = NO;
        }
        else {
            ret = [self.mvideo previewLocalCamera:front render:self.video_bg];
            if (ret == AVD_Success) {
                self.video_previewed = YES;
                AVDDeviceId curid = [self.mvideo getCamera: [self.mvideo getCurrentCameraType] ].id;
                [self.video_bg setVideo:curid title:@"preview test" isSwitch:NO isRecord:NO isLive:NO];
                if (self.mixerav) {
                    [self.mixerav startWithMainVideoId:curid];
                }
            }else {
                NSLog(@"openCloseLocacamera, ret=%d", ret);
            }
        }
        return YES;
    }
    if ([self.mvideo ispublishLocalCamera]) {
        [self.mvideo unpublishLocalCamera];
        isOpen = NO;
    } else {
        [self.mvideo publishLocalCamera:front];
    }
    return isOpen;
}

- (void)setBackground:(NSString*)imagename {
    [self.video_bg setBackground:imagename];
}

- (void)switch2BigRender:(NSNotification *)aNotification{
    AVDDeviceId videobg = [[aNotification userInfo] objectForKey:@"DeviceId"];
    if (nil == videobg || [videobg isEqualToString:@""]) {
        NSLog(@"switch2BigRender, deviceId is null");
        return ;
    }
    AVDDeviceId videoup = self.video_bg.deviceId;
    VideoViewCell *upcell = [self.videos_up getRenderByVideoId:videobg];
    [self detachVideoView:upcell];
    [self detachVideoView:self.video_bg];
    if (nil == videobg || [videobg isEqualToString:@""]) {
        return ;
    }
    [self attachRender:videobg];
    
    if (nil == videoup || [videoup isEqualToString:@""]) {
        return ;
    }
    [self attachRender:videoup];
}
- (void)userRemoteCmd:(NSNotification *)aNotification{
    NSString *type = [[aNotification userInfo] objectForKey:@"type"];
    NSString *userId = [[aNotification userInfo] objectForKey:@"userId"];
    NSNumber *status = [[aNotification userInfo] objectForKey:@"status"];
    if (nil == type || nil == userId || nil == status) {
        NSLog(@"userRemoteCmd, type:%@ or userId:%@ status:%@ is null", type, userId, status);
        return ;
    }
    if ([self.mvideo isSelfUser:userId]) {
        return ;
    }
    if      ([type isEqualToString:@"video"]) {
        AVDCamera *camera = [self.mvideo getCameraByUserId:userId];
        if      (1 == status.integerValue) {
            [self.mvideo remotecmdPublishCamera:camera];
        }
        else if (2 == status.integerValue) {
            [self.mvideo remotecmdUnpublishCamera:camera.id];
        }
    }
    else if ([type isEqualToString:@"audio"]) {
        if      (1 == status.integerValue) {
            [self.maudio remotecmdOpenMicrophone:userId];
        }
        else if (2 == status.integerValue) {
            [self.maudio remotecmdCloseMicrophone:userId];
        }
    };
}

//
- (void) defaultSelectVideo4Show {
    if (0 == self.mvideo.publishedCameras.count
        && 0 == self.mscreen.publishedScreens.count) {
        return ;
    }
    NSMutableArray *shows = [[NSMutableArray alloc] initWithArray:self.mscreen.publishedScreens];
    [shows addObjectsFromArray:self.mvideo.publishedCameras];
    NSArray *displayedVideos = [self getShowVideos];
    if (displayedVideos.count >0) {
        for(NSString* displayedId in displayedVideos) {
            for(AVDVideoDevice* dev in shows) {
                if ([dev.id isEqualToString:displayedId]) {
                    [shows removeObject:dev];
                    break;
                }
            }
        }
    }
    NSUInteger needSub = self.videos_up.videoMax +1 -displayedVideos.count;
    while(shows.count >needSub) {
        [shows removeLastObject];
    }
    for(AVDVideoDevice* dev in shows) {
        [self subscribeVideo:dev isScreen:![dev isKindOfClass:[AVDCamera class]]];
    }
}
- (NSArray *) getShowVideos {
    NSMutableArray *displayedVideos = [[NSMutableArray alloc] initWithArray:self.videos_up.videoIds];
    if ([self.video_bg hasVideo]) {
        [displayedVideos addObject:self.video_bg.deviceId];
    }
    return displayedVideos;
}
- (void) unsubscribeVideoAndReshow:(AVDVideoDevice*)from isScreen:(BOOL)isScreen {
    [self unsubscribeVideo:from isScreen:isScreen];
    [self defaultSelectVideo4Show];
}

- (void) subscribeVideo:(AVDVideoDevice*)from isScreen:(BOOL)isScreen {
    if (![self hasFreeRender]) {
        return ;
    }
    if ([self.mvideo isSelfDevice:from.id]) {
        [self attachRender:from.id];
    }
    else {
        if (isScreen) {
            [self.mscreen subscribe:from.id];
        }
        else {
            [self.mvideo subscribe:from.id];
        }
    }
}
- (void) unsubscribeVideo:(AVDVideoDevice*)from isScreen:(BOOL)isScreen {
    [self detachRender:from.id];
    if (![self.mvideo isSelfDevice:from.id]) {
        if (isScreen) {
            [self.mscreen unsubscribe:from.id];
        }
        else {
            [self.mvideo unsubscribe:from.id];
        }
    }
}

//
- (BOOL)hasFreeRender {
    if (![self.video_bg hasVideo]) {
        return YES;
    }
    return [self.videos_up hasFreeRender];
}
- (VideoViewCell *)getFreeRenderForVideoId:(AVDDeviceId)videoId {
    if (![self.video_bg hasVideo]) {
        return self.video_bg;
    }
    return [self.videos_up getFreeRenderForVideoId:videoId];
}
- (VideoViewCell *)getRenderByVideoId:(AVDDeviceId)videoId {
    if ([self.video_bg hasVideo] && [self.video_bg.deviceId isEqualToString:videoId]) {
        return self.video_bg;
    }
    return [self.videos_up getRenderByVideoId:videoId];
}
- (void) attachRender:(AVDDeviceId)deviceId {
    VideoViewCell *cell = [self getFreeRenderForVideoId:deviceId];
    if (nil == cell) {
        NSLog(@"VideosView, attachRender, no free cell. videoId:%@", deviceId);
        return ;
    }
    BOOL vswitch = [self.mvideo isSelfDevice:deviceId];
    BOOL vrec = [self isRecord:deviceId];
    BOOL vliv = [self isLive:deviceId];
    [cell setVideo:deviceId title:[self.mvideo getOwnerName:deviceId] isSwitch:vswitch isRecord:vrec isLive:vliv];
    if (cell == self.video_bg) {
//        [self.video_bg setScalingType:Scale_Aspect_Full];
//        for (AVDVideoDevice* item in self.mscreen.publishedScreens) {
//            if ([item.id isEqualToString:deviceId]) {
//                [self.video_bg setScalingType:Scale_Aspect_Fit];
//                break;
//            }
//        }
    }
    else {
        self.videos_up.hidden = NO;
        [self layoutSubviews];
    }
    [self.mvideo attachRenderWithDeviceId:deviceId render:cell];
}

- (void) detachRender:(AVDDeviceId)deviceId {
    VideoViewCell *cell = [self getRenderByVideoId:deviceId];
    if (nil == cell) {
        return ;
    }
    [self detachVideoView:cell];
    if (cell == self.video_bg && [self.videos_up hasVideos]) { // switch up to bg
        [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationGestureRecognizerSelect object:self userInfo:[NSDictionary dictionaryWithObject:self.videos_up.videoIds.firstObject forKey:@"DeviceId"]];
    }
}
- (void) detachVideoView:(VideoViewCell*)cell {
    if (nil == cell || ![cell hasVideo]) {
        return ;
    }
    [self.mvideo detachRenderWithRender:cell];
    if (cell != self.video_bg) {
        [self.videos_up removeCell:cell];
    }
    [cell setVideo:nil title:nil isSwitch:NO isRecord:NO isLive:NO];
    if (![self.videos_up hasVideos]) {
        self.videos_up.hidden = YES;
    }
    [self layoutSubviews];
}

#pragma mark --video delegate 发布视频时回调这些方法
- (void) onCameraStatusNotify:(enum AVDDeviceStatus)status deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onCameraStatusNotify, status:%u, from:%@", status, fromId);
    if ([self.mvideo isSelfDevice:fromId] || ds_published != status) {
        return ;
    }
    if ([self hasFreeRender]) {
        return ; // subscribe in onPublishCameraNotify
    }
    // modified for qiniu data callback
    if (self.mixerav || [self.mixerav hasOverlaySpace]) {
        [self.mvideo subscribe:fromId];
    }
}
- (void) onCameraDataNotify:(NSInteger)level description:(NSString *)description deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onCameraDataNotify, result:%lu fromId:%@", (unsigned long)level, fromId);
}
- (void) onPublishCameraNotify:(AVDCamera*)item {
    NSLog(@"VideosView, onPublishCameraNotify, camera:%@", item.id);
    [self subscribeVideo:item isScreen:NO];
    // modified for qiniu data callback
    if (self.mixerav && [self.mvideo isSelfDevice:item.id]) {
        [self.mixerav listenData];
    }
}
- (void) onUnpublishCameraNotify:(AVDCamera*)item {
    NSLog(@"VideosView, onUnpublishCameraNotify, camera:%@", item.id);
    [self unsubscribeVideoAndReshow:item isScreen:NO];
    // modified for qiniu data callback
    if (self.mixerav && [self.mvideo isSelfDevice:item.id]) {
        [self.mixerav stopExport];
    }
}
#pragma mark --screen delegate 发布屏幕时回调这些方法
- (void) onScreenStatusNotify:(enum AVDDeviceStatus)status deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onScreenStatusNotify, status:%u, from:%@", status, fromId);
}
- (void) onScreenDataNotify:(NSInteger)level description:(NSString *)description deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onScreenDataNotify, result:%lu fromId:%@", (unsigned long)level, fromId);
}
- (void) onPublishScreenNotify:(AVDVideoDevice*)item {
    NSLog(@"VideosView, onPublishScreenNotify, screen:%@", item.id);
    [self subscribeVideo:item isScreen:YES];
}
- (void) onUnpublishScreenNotify:(AVDVideoDevice*)item {
    NSLog(@"VideosView, onUnpublishScreenNotify, screen:%@", item.id);
    [self unsubscribeVideoAndReshow:item isScreen:YES];
}
#pragma mark --screen video delegate common
- (void) onSubscribeResult:(AVDResult)result deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onSubscribeResult, result:%lu fromId:%@", (unsigned long)result, fromId);
    if (AVD_Success != result) {
        return ;
    }
    [self attachRender:fromId];
    // modified for qiniu data callback
    if (nil != self.videostatusdelegate) {
        [self.videostatusdelegate onVideo:fromId subscribed:YES];
    }
}
- (void) onUnsubscribeResult:(AVDResult)result deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onUnsubscribeResult, result:%lu fromId:%@", (unsigned long)result, fromId);
    // modified for qiniu data callback
    if (nil != self.videostatusdelegate) {
        [self.videostatusdelegate onVideo:fromId subscribed:NO];
    }
}
- (void) onPublishLocalResult:(AVDResult)result deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onPublishLocalResult, result:%lu fromId:%@", (unsigned long)result, fromId);
}
- (void) onUnpublishLocalResult:(AVDResult)result deviceId:(AVDDeviceId)fromId {
    NSLog(@"VideosView, onUnpublishLocalResult, result:%lu fromId:%@", (unsigned long)result, fromId);
    self.webview.hidden = YES;

}

#pragma mark --record delegate imp
- (void) onCreateUserRecord:(AVDResult)result info:(AVDRecordInfo*)info {
    NSLog(@"onCreateUserRecord,result=%lu,info=%@", (unsigned long)result, info);
    if (nil == self.curRecord) {
        return ;
    }
    VideoViewCell* render = [self getRenderByUserId:self.curRecord.userId];
    if (nil != render) {
        render.recordId = ((AVD_Success == result) ? info.id : nil);
    }
    self.curRecord = (AVD_Success == result) ? info : nil;
    if (AVD_Success != result) {
        
    }
}
- (void) onStopRecord:(AVDResult)result id:(AVDRecordId)infoid {
    NSLog(@"onStopRecord,result=%lu,recordId=%@", (unsigned long)result, infoid);
    self.curRecord = nil;
}
- (void) onRecordInfo:(AVDResult)result info:(AVDRecordInfo*)info {
}
- (void) onRecordInfos:(AVDResult)result total:(NSInteger)total begin:(NSInteger)begin infos:(NSArray*)items {
}
- (void) onDeleteRecord:(AVDResult)result id:(AVDRecordId)infoid {
}
#pragma mark --live delegate imp
- (void) onCreateUserLive:(AVDResult)result info:(AVDLiveInfo*)info {
    NSLog(@"onCreateUserRecord,result=%lu,info=%@", (unsigned long)result, info);
    if (nil == self.curLive) {
        return ;
    }
    VideoViewCell* render = [self getRenderByUserId:self.curLive.userId];
    if (nil != render) {
        render.liveId = ((AVD_Success == result) ? info.id : nil);
    }
    self.curLive = (AVD_Success == result) ? info : nil;
    if (AVD_Success != result) {
        
    }
    self.needLoad = YES;
}
- (void) onStopLive:(AVDResult)result id:(AVDRecordId)infoid {
}
- (void) onLiveInfo:(AVDResult)result info:(AVDLiveInfo*)info {
}
- (void) onLiveInfos:(AVDResult)result total:(NSInteger)total begin:(NSInteger)begin infos:(NSArray*)items {
}
- (void) onDeleteLive:(AVDResult)result id:(AVDLiveId)infoid {
    NSLog(@"onDeleteLive,result=%lu,liveId=%@", (unsigned long)result, infoid);
    self.curLive = nil;
    self.webview.hidden = YES;
}

- (void) dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark --live and record video imp
- (NSString*) getRecOrLiveName:(AVDDeviceId)videoId type:(NSString*)type {
    NSDate* nowdate=[NSDate date];
    NSDateFormatter *dateformatter=[[NSDateFormatter alloc] init];
    [dateformatter setDateFormat:@"YYYY-MM-dd_HH:mm:ss"];
    NSString* locationString=[dateformatter stringFromDate:nowdate];
    
    NSString* name = [self.mvideo getOwnerName:videoId];
    name = [name stringByAppendingFormat:@"_%@_%@", type, locationString];
    return name;
}
- (BOOL) isLive:(AVDDeviceId)videoId {
    if (![self.mvideo isSelfDevice:videoId]) {
        return NO;
    }
    NSString* codec = [N2MSetting getVideoCodecOption];
    if (![codec isEqualToString:@"h264"]) {
        return NO;
    }
    AVDRoom* room = [AVDRoom obtain:[N2MSetting getCurrentRoomId]];
    if (nil == room) {
        return NO;
    }
    if (![[room getRoomInfo] isMcu]) {
        return NO;
    }
    return YES;
}
- (BOOL) isRecord:(AVDDeviceId)videoId {
    if (![self.mvideo isSelfDevice:videoId]) {
        return NO;
    }
    AVDRoom* room = [AVDRoom obtain:[N2MSetting getCurrentRoomId]];
    if (nil == room) {
        return NO;
    }
    if (![[room getRoomInfo] isMcu]) {
        return NO;
    }
    return YES;
}
- (enum AVDRecordVideoType) getH264Stream:(AVDDeviceId)videoId {
    return rv_main;
}
- (VideoViewCell*) getRenderByUserId:(AVDUserId)userId {
    if ([self.video_bg hasVideo]) {
        AVDUserId ownid = [self.mvideo getOwnerId:self.video_bg.deviceId];
        if ([userId isEqualToString:ownid]) {
            return self.video_bg;
        }
    }
    NSUInteger location = 0;
    for(; location<[self.videos_up.videoIds count]; ++location) {
        VideoViewCell* item = [self getRenderByVideoId:self.videos_up.videoIds[location]];
        if (nil == item) {
            continue;
        }
        AVDUserId ownid = [self.mvideo getOwnerId:item.deviceId];
        if ([userId isEqualToString:ownid]) {
            return item;
        }
    }
            return nil;
}

- (void) stopLiveRecord {
    NSLog(@"stopLiveRecord,live=%@,record=%@", self.curLive, self.curRecord);
    if (nil != self.curLive) {
        [self.mlive deleteLive:self.curLive.id];
        self.curLive = nil;
    }
    if (nil != self.curRecord) {
        [self.mrecord stopRecord:self.curRecord.id];
        self.curRecord = nil;
    }
    // modified for qiniu data callback
    if (self.mixerav) {
        [self.mixerav stopExport];
    }
}

@end
